package com.hzs.third.pay.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hzs.common.domain.third.pay.TOnlineCard;
import com.hzs.third.pay.mapper.TOnlineCardMapper;
import com.hzs.third.pay.service.ITOnlineCardService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 在线银行卡明细 服务实现类
 */
@Service
public class TOnlineCardServiceImpl extends ServiceImpl<TOnlineCardMapper, TOnlineCard> implements ITOnlineCardService {

    @Override
    public List<TOnlineCard> queryBankList(Long pkMember, String customerNum, Integer pkCountry) {
        return baseMapper.queryBankList(pkMember, customerNum, pkCountry);
    }

    @Override
    public TOnlineCard checkUserBankExist(Long pkMember, String bankNo, String customerNum, Integer pkCountry) {
        return baseMapper.checkUserBankExist(pkMember, bankNo, customerNum, pkCountry);
    }

    @Override
    public TOnlineCard queryCardByCode(String code, Long pkMember, String customerNum, Integer pkCountry) {
        return baseMapper.queryCardByCode(code, pkMember, customerNum, pkCountry);
    }

    @Override
    public int updateBankInfo(TOnlineCard tOnlineCard) {
        return baseMapper.updateBankInfo(tOnlineCard);
    }

    @Override
    public int unBindBank(String code, Long pkMember, String customerNum) {
        return baseMapper.unBindBank(code, pkMember, customerNum);
    }

}
